/*Copyright (C) 2005  Francois DEVIC
            http://www.devic.net
            
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Steet, Fifth Floor,
Boston, MA 02110-1301, USA

*/

/** @file param.cpp
  * @brief fichier du corps de la classe CParam
  * @author F.DEVIC
  * @date 18 aout 2005
  **/
  
#include "param.h"
#include <fstream>
#include <sstream>
#include <QDate>
#include <QFile>
#include <QDir>
#include <cstdlib>
#include <iostream>

using namespace std;

/** variables statiques*/
QString     CParam::m_Smaya_path;
QString     CParam::m_Smr_path;
vectShader CParam::m_Vvect_Shader;


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** @fn void CParam::writeMayaPath(const QString &maya_Path)
   * ecrit le path de maya dans le fichier de configuration
*/
void CParam::writeMayaPath(const QString & maya_Path)
{
     char tmp[512];
    std::ifstream iparam( "init.txt");
    std::ofstream outparam;

    std::stringstream buffer; // variable contenant l'intgralit du fichier
    if ( iparam ) // ce test choue si le fichier n'est pas ouvert
    {
    
        // copier l'intgralit du fichier dans le buffer
        buffer << iparam.rdbuf();
        
        iparam.close();
        buffer.getline(tmp,512);
        if(strcmp(tmp,"[MAYA_PATH]"))
        {
        // //cout<<"ERREUR lecture param"<<endl ;                            
        }
        else
        {
        
         outparam.open("init.txt", ios::out|ios::trunc);
         outparam<<tmp<<endl;
        
         buffer.getline(tmp,512);
         outparam<<maya_Path.toStdString()<<endl;
         while(!buffer.eof())
         {
           buffer.getline(tmp,512);
           outparam<<tmp<<endl;
         }
         outparam.close();         
        }
    }
    else
    {
    ////cout<<"ouverture du fichier"<<endl;
     outparam.open ( "init.txt" );  
     if(outparam.is_open())
     {
      //  //cout<<"fichier ouvert"<<endl;
         outparam<<"[MAYA_PATH]"<<endl;
         outparam<<maya_Path.toStdString()<<endl;
         outparam.close();
      }
      else
      {
      // //cout<<"probleme d'ouverture"<<endl;
      }
    }
        
    
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////  
  
  
  
  
  
  
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////  
/** @fn void CParam::writeMentalRayPath(const QString & mr_Path)
   * ecrit le path de mental Ray dans le fichier de configuration
*/
void CParam::writeMentalRayPath(const QString & mr_Path)
{
////cout<< "CParam::writeMentalRayPath"<<endl;
     char tmp1[512],tmp2[512],tmp3[512];
    std::ifstream iparam( "init.txt");
    std::ofstream outparam;

    std::stringstream buffer; // variable contenant l'intgralit du fichier
    if ( iparam ) // ce test choue si le fichier n'est pas ouvert
    {
    
        // copier l'intgralit du fichier dans le buffer
        buffer << iparam.rdbuf();
        
        iparam.close();
        buffer.getline(tmp1,512);
        
        if(strcmp(tmp1,"[MAYA_PATH]"))
        {
         ////cout<<"ERREUR lecture param"<<endl ;                            
         return;
        }
        //le chemin de Maya
        buffer.getline(tmp2,512);

        buffer.getline(tmp3,512);
        if(strcmp(tmp3,"[MR_PATH]"))
        {
             outparam.open ( "init.txt", ios::out|ios::app );       
             if(outparam.is_open())
             {
           //   //cout<<"fichier ouvert"<<endl;
              outparam<<"[MR_PATH]"<<endl;
              outparam<<mr_Path.toStdString()<<endl;
              outparam.close();
              }
        }
        else
        {
            outparam.open("init.txt",ios::out|ios::trunc);
            if(outparam.is_open())
            {
             outparam<<tmp1<<endl<<tmp2<<endl<<tmp3<<endl<<mr_Path.toStdString()<<endl;
             //on lit l'ancienne valeur de Mental RAY
             buffer.getline(tmp3,512);
             //on lit la suite du fichier
             while(buffer.getline(tmp3,512))
             {
                outparam<<tmp3<<endl;
             }                      
             outparam.close();
            }
             
        }
      }
      else
      {
       //cout<<"probleme d'ouverture"<<endl;
      }
}  
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////  
  



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////  
bool CParam::addShader()
{
     m_Vvect_Shader.resize(m_Vvect_Shader.size()+1);
     
     return m_Vvect_Shader[m_Vvect_Shader.size()-1].writeShader();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////









/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
   * @fn bool CParam::init()
   * @brief initialisation des parametres
*/
bool CParam::init()
{
 QString tmp;
 QFile paramFile("init.txt");
 bool res = true;

 if(paramFile.open(QIODevice::ReadOnly | QIODevice::Text))
  {
    QTextStream param(&paramFile);
    //lecture du chemin de Maya
    tmp = param.readLine();
    if(!param.atEnd() && tmp.indexOf("[MAYA_PATH]")>=0)
    {
        tmp = param.readLine();
        m_Smaya_path = tmp;       
    }
    else
    {
        paramFile.close();
        return false;    
    }                          
   
   //lecture du chemin de Mental Ray
  tmp = param.readLine();

   if(!param.atEnd() && tmp.indexOf("[MR_PATH]")>=0)
   {
        tmp = param.readLine();
        
        m_Smr_path = tmp;       
        
        if(saveMaya_rayrc())
        {

        }
        else
        {

        }
   }
   else
   {
    ////cout<<"erreur de lecture : chemin de mental Ray inexistant"<<endl;
    paramFile.close();
    return false;    
   }
   //lecture de tous les shaders
   tmp = param.readLine();
   while(!param.atEnd())
   {
       //on a trouve un shader
       if(tmp.indexOf("[")>=0)
       {
          tmp.truncate(tmp.size()-1); //on vire le "]"
          tmp=tmp.right(tmp.size()-1); //on vire le "["
          m_Vvect_Shader.resize(m_Vvect_Shader.size()+1);

          m_Vvect_Shader[m_Vvect_Shader.size()-1].readShader(param,tmp);
          
       }
       tmp = param.readLine();
   }
  paramFile.close();                      
 }
 else
 {
    res = false; 
 }
 
 return res;     
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
  * @fn void CParam::displayParam()
  * @brief affiche tous les paramtres
*/

void CParam::displayParam()
{
     
     if(m_Vvect_Shader.size()>0)
     {
       for(int i=0;i<m_Vvect_Shader.size();++i)
       {
          m_Vvect_Shader[i].display();        
       }
     }
     else
     {
     }
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** @fn bool saveMaya_rayrc()
  * @brief sauvegarde le fichier maya.rayrc
  * @return true si a c'est bien pass
  *         false sinon
**/
bool CParam::saveMaya_rayrc()
{
      //rcupration de la date et du jour
     QDate date = QDate::currentDate();
     QString maya_rayrc;
     QString new_maya_rayrc;
     QString tmp;

     //construction des chemins
     maya_rayrc = m_Smr_path ;
     maya_rayrc += "\\maya.rayrc";

     //construction du nouveau nom
     new_maya_rayrc = QDir::currentPath();
     new_maya_rayrc += "\\maya.rayrc";

     //construction de la date
     tmp = QString("%1_%2_%3").arg(date.year(),4,10,QChar('0')).arg(date.month(),2,10,QChar('0')).arg(date.day(),2,10,QChar('0'));
     new_maya_rayrc +=tmp;

     //copie du fichier
     return QFile::copy ( maya_rayrc, new_maya_rayrc ) ;

}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** 
  * @fn void CParam::copyShader()
  * @brief copie les shaders au bon endroit
  *  affichage de tous les shaders
  *  on demande quel type de shade on veut copie
*/
void CParam::copyShader()
{
     QString tmp;
     int res = 1;

     if(res>0 && res<=m_Vvect_Shader.size())
     {
          m_Vvect_Shader[res-1].copyShader(tmp,m_Smr_path,QString(),QString());    
     }
}

/** 
  * @fn bool CParam::copyShader(const QString & shaderPath, const QString &link,const QString &mi,int num_shader=0)
  * @brief copie les shaders au bon endroit
  * @param shaderPath : chemin du shader a ajouter
  * @param num_shader : numero du shader
*/
bool CParam::copyShader(const QString & shaderPath, const QString &link,const QString &mi,int num_shader)
{
     bool res = true;
     if(num_shader>=0 && num_shader<m_Vvect_Shader.size())
     {
          res &=m_Vvect_Shader[num_shader].copyShader(shaderPath,m_Smr_path,link,mi);    
     }
     return res;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CParam::setChemin(int num_shader,int num_chemin,const QString & ext,const QString & path, bool mandatory)
{
 if(num_shader>=0 && num_shader<m_Vvect_Shader.size())
 {
     m_Vvect_Shader[num_shader].setChemin(num_chemin,ext,path,mandatory);
 }     
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
  * @fn void CParam::saveParam()
  * @brief sauvegarde tous les parametres
*/
void CParam::saveParam()
{
     QFile saveFile("init.txt");
     
     if(saveFile.open(QIODevice::WriteOnly | QIODevice::Text))
     {
      QTextStream save(&saveFile);
      save<<"[MAYA_PATH]"<<endl;
      save<<m_Smaya_path<<endl;
      save<<"[MR_PATH]"<<endl;
      save<<m_Smr_path<<endl;
      
      
      for(int i=0;i<m_Vvect_Shader.size();++i)
      {
       m_Vvect_Shader[i].saveShader(save);        
      }
      saveFile.close();
     }
}
