/*Copyright (C) 2005  Francois DEVIC
            http://www.devic.net
            
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Steet, Fifth Floor,
Boston, MA 02110-1301, USA

*/

/** @file shader.cpp
  * @brief fichier du corps de la classe CShader
  * @author F.DEVIC
  * @date 18 aout 2005
  **/
  

#include <fstream>
#include <sstream>
#include <QObject>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QMessageBox>
#include <QString>

#include "shader.h"

using namespace std;


/**
 * @fn bool CShader::writeShader()
 * ecrit le shader dans le fichier de configuration
*/
bool CShader::writeShader()
{
     char tmp1[512],tmp2[512],tmp3[512],tmp4[512];
     bool res=true;
     int nb_ext;
     
    std::ifstream iparam( "init.txt");
    QFile outparamFile("init.txt");

    std::stringstream buffer; // variable contenant l'intgralit du fichier
    if ( iparam ) // ce test choue si le fichier n'est pas ouvert
    {
    
        // copier l'intgralit du fichier dans le buffer
        buffer << iparam.rdbuf();
        
        iparam.close();
        buffer.getline(tmp1,512);
        
        if(strcmp(tmp1,"[MAYA_PATH]"))
        {
         return false;
        }
        //le chemin de Maya
        buffer.getline(tmp2,512);

        buffer.getline(tmp3,512);
        if(strcmp(tmp3,"[MR_PATH]"))
        {
         return false;
        }

        buffer.getline(tmp4,512);

        m_Sname=QString("Shader Mental Ray");
        nb_ext=5;
        //initialisation du nombre d'extension:
        m_Vchemin.resize(nb_ext);
          if(outparamFile.open(QIODevice::WriteOnly | QIODevice::Text))
          {
              QTextStream outparam(&outparamFile);
              saveShader(outparam);
              outparamFile.close();
              res =true;
              }
        
    }
    else
    {

     res =false;    
    }
    
    
 return res;     
}
////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////
/**
  * @fn void CShader::saveShader(QTextStream & outparam)
  * @brief sauvegarde du shader dans le flux deja ouvert
  * @param outparam un QTextStream deja ouvert
*/
void CShader::saveShader(QTextStream & outparam)
{
    outparam<<"["<<m_Sname<<"]"<<endl;
    outparam<<m_Vchemin.size()<<endl;
    
    
    for(int i = 0; i< m_Vchemin.size();++i)
    {
        m_Vchemin[i].write(outparam);   
    }    
}


////////////////////////////////////////////////////////////////////////////////////////
/**
 * @fn void CShader::readShader(QTextStream & inparam, const QString &name)
 * @brief lit un shader dans le fichier
 * @param inparam fichier o lire les paramtres
 * @param name nom du shader
 * @warning le fichier doit etre ouvert au bon endroit (c'est  dire aprs les crochets)
 * @warning aucune donnee ne sera verifiee : il faut donc qu'lle soit valide
*/
void CShader::readShader(QTextStream & inparam, const QString &name)
{
     QString tmp;
     int nb_ext;
   //le nom
   m_Sname = name; 
   //recuperation du nombres d'extension 
   tmp=inparam.readLine();
   nb_ext=tmp.toInt();
   //on efface tout le vecteur
   m_Vchemin.clear();
   m_Vchemin.resize(nb_ext);
   for(int i=0;i<nb_ext;++i)
   {
    m_Vchemin[i].readChemin(inparam);        
   }
}
////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////
/**
 * @fn void CShader::display()
 * @brief affiche le contenu de CShader
*/

void CShader::display()
{
     for (int i =0;i<m_Vchemin.size();++i)
     {
         m_Vchemin[i].display();    
     }
}
////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////
/**
  * @fn bool CShader::copyShader(const QString & shader_path,const QString & renderer_path,const QString &link,const QString &mi)
  * @brief copie les fichiers des shaders au bon endroit
  * @param shader_path l'endroit de base du shader
  * @param renderer_path chemin du renderer pour faire la sauvegarde
*/
bool CShader::copyShader(const QString & shader_path,const QString & renderer_path,const QString &link,const QString &mi)
{
     QString tmp;
     QString tmp_fichier;
     QString pathCopy;
     bool res=true;
     bool copy;

    if(fillMayaRayrc(renderer_path,link,mi))
    {
       res=true;                                         
    }
    else
    {
       QMessageBox::warning(NULL,QObject::tr("Erreur d'ecriture"),QObject::tr("Erreur lors de l'criture dans maya.rayrc"));
       res=false;    
    }
    for(int i=0;i<m_Vchemin.size();++i)
    {
         tmp[0]='\0';

         res &= !(getFileNameByExtension(m_Vchemin[i].getExtension(),shader_path,tmp,tmp_fichier).isEmpty());

         pathCopy = m_Vchemin[i].getPath();
         pathCopy+="\\";
         pathCopy+=tmp_fichier;


         copy=QFile::copy(tmp,pathCopy);
         if(!copy)
         {
                  //pour savoir le fichier est obligatoire ou non
                  if(m_Vchemin[i].isMandatory())
                  {
                           QMessageBox::critical(NULL,QObject::tr("Erreur de copie de fichier"),QObject::tr("Erreur lors de la copie du fichier : ")+tmp_fichier,
                              QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
                              res= false;
                  }
                  else
                  {
                      QMessageBox::warning(NULL,QObject::tr("Erreur de copie de fichier"),QObject::tr("Erreur lors de la copie du fichier : ")+tmp_fichier+QObject::tr("\n Fichier non obligatoire"));
                      res&=true;
                  }
                              
         }
    } 
    
    return res;
}
////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////
/**
  * @fn bool CShader::fillMayaRayrc(const QString & mr_path,const QString &link,const QString &mi)
  * @brief pour remplir le fichier maya.rayrc comme il faut
  * @param mr_path chemin de mental ray
*/
bool CShader::fillMayaRayrc(const QString & mr_path,const QString &link,const QString &mi)
{
    char tmp[512];
    bool linkok=false;
    bool miok=false;
    bool res=false;
    QString mayarayrc;
    
    
    mayarayrc=mr_path;
    mayarayrc+="\\maya.rayrc";
 
    std::ifstream iparam( mayarayrc.toStdString().data());
    QFile outparamFile(mayarayrc);

    std::stringstream buffer; // variable contenant l'intgralit du fichier
    if ( iparam ) // ce test choue si le fichier n'est pas ouvert
    {
    
        // copier l'intgralit du fichier dans le buffer
        buffer << iparam.rdbuf();
        iparam.close();
        
        if(outparamFile.open(QIODevice::WriteOnly | QIODevice::Text))
        {
            QTextStream outparam(&outparamFile);
            while(buffer.getline(tmp,512))
            {
               if(strstr(tmp,"\tlink")&&!linkok)
               {
                outparam<<"\t"<<link<<endl;                              
                linkok=true;
               }
               if(strstr(tmp,"\tmi")&&!miok)
               {
                outparam<<"\t"<<mi<<endl;
                miok=true;                           
               }
               outparam<<tmp<<endl;                  
            }
            outparamFile.close();
        }    
     } 
     res=linkok & miok;
    return res; 
}


////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////
/**
  * @fn char * CShader::getFileNameByExtension(const char * extension,const char * path,char *resultat,char * fichier)
  * @brief recuperer le chemin exact du fichier ayant l'extension extension  partir de path
  * @param extension extension du fichier  trouver
  * @param path cheminde dpart
  * @param resultat le chemin
*/
QString CShader::getFileNameByExtension(const QString & extension,const QString & path,QString & resultat,QString & fichier)
{
     bool re=true;
     bool find=false;
     QString chemin;

     QString res;
     QString tmp;
     QString tmp_fichier;
     QFileInfoList listeFichier;
     QDir rep(path);

     int nb_dir = 0; //nombre de repertoire parcouru
     
     chemin = path;

    listeFichier = rep.entryInfoList();     
    
    if(listeFichier.size() ==  0)
    {
            return QString();
    }
    do {
    
              tmp_fichier = listeFichier[nb_dir].fileName();         

            if(re && tmp_fichier.indexOf(extension)>0 && (!listeFichier[nb_dir].isDir()))
            {
             res=chemin;
             res+="\\";
             res+=tmp_fichier;
             fichier=tmp_fichier; 
             re=false;
             find=true;
            }
            
            nb_dir++;
    } while(re && nb_dir<listeFichier.size());

    nb_dir = 0;
    
    re=true;
    //recuperation des repertoires
    if(!find)
    {
    do {
    
            tmp_fichier = listeFichier[nb_dir].fileName();

            if(listeFichier[nb_dir].isDir() && (tmp_fichier!="..") && (tmp_fichier!="."))
            {
             chemin=path;
             chemin+="\\";
             chemin+=tmp_fichier;

             //on va chercher dans ce repertoire
             getFileNameByExtension(extension,chemin,tmp,tmp_fichier);
             //si on a trouve
             if(tmp_fichier.indexOf(extension)>0)
             {
              res=tmp;

              fichier=tmp_fichier;

               re=false;                          
             }
            }
            nb_dir++;
            
    } while(re &&nb_dir<listeFichier.size());

    }

    //copie du rsultat
    resultat=res;
    return resultat;
}
////////////////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////////
void CShader::setChemin(int num_chemin,const QString & ext,const QString & path, bool mandatory)
{
 if(num_chemin>=0 && num_chemin<m_Vchemin.size())
 {
  m_Vchemin[num_chemin].setExtension(ext);
  m_Vchemin[num_chemin].setPath(path);
  m_Vchemin[num_chemin].setMandatory(mandatory);
 }
}
