/****************************************************************************
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
/*Copyright (C) 2005  Francois DEVIC
            http://www.devic.net
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Steet, Fifth Floor,
Boston, MA 02110-1301, USA

*/
#include <QtGui>
#include <QFileDialog>
#include <QPushButton>
#include <QTextEdit>
#include <QTextBrowser>
#include "tabdialog.h"
#include "param.h"


////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

/**
  * @fn TabDialog::TabDialog( QWidget *parent)
  * @brief constructeur de TabDialog
*/
TabDialog::TabDialog( QWidget *parent)
    : QDialog(parent)
{

    tabWidget = new QTabWidget;
    tabWidget->addTab(new ShaderTab, tr("Shader"));
    tabWidget->addTab(new ConfigTab, tr("Configuration"));
    tabWidget->addTab(new CreditTab, tr("Crdits"));

    QPushButton *okButton = new QPushButton(tr("OK"));
    QPushButton *cancelButton = new QPushButton(tr("Cancel"));

    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabWidget);
    mainLayout->addLayout(buttonLayout);
    setLayout(mainLayout);

    setWindowTitle(tr("ShaderCopy"));
}



////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////



ShaderTab::ShaderTab( QWidget *parent)
    : QWidget(parent)
{
    QGroupBox *shaderGroup = new QGroupBox(tr("Shaders"));
    
    m_CshaderCombo = new QComboBox();
    for(int i=0;i<CParam::getShaders().size();++i)
    {
     m_CshaderCombo->addItem(CParam::getShaders()[i].getName());
    }
    m_CshaderCombo->setEnabled( false );
    
    QLabel *shaderLabelCombo = new QLabel(tr("Type de Shader"));
    QLabel *shaderLabel = new QLabel(tr("Chemin de votre Shader"));
    QLabel *shaderLabelLink = new QLabel(tr("La ligne Link"));
    QLabel *shaderLabelMi = new QLabel(tr("La ligne mi"));
    
    m_LEshader_Edit = new QLineEdit();
    m_LElink_Edit = new QLineEdit();
    m_LEmi_Edit = new QLineEdit();
    
    QPushButton *ShaderPath = new QPushButton("...");
    ShaderPath->setMaximumSize(35,20);
    
    connect(ShaderPath, SIGNAL(clicked()),
            this, SLOT(FindShaderPath()));

    QHBoxLayout * shaderHLayout = new QHBoxLayout;
    shaderHLayout->addWidget(m_LEshader_Edit);
    shaderHLayout->addWidget(ShaderPath);

    QVBoxLayout *shaderVLayout = new QVBoxLayout;
    shaderVLayout->addWidget(shaderLabelCombo);
    shaderVLayout->addWidget(m_CshaderCombo);
    
    shaderVLayout->addWidget(shaderLabel);
    shaderVLayout->addLayout(shaderHLayout);
    
    shaderVLayout->addWidget(shaderLabelLink);
    shaderVLayout->addWidget(m_LElink_Edit);
    shaderVLayout->addWidget(shaderLabelMi);
    shaderVLayout->addWidget(m_LEmi_Edit);
    
    QPushButton * saveShaderButton = new QPushButton(tr("Installer le Shader"));
   	connect(saveShaderButton, SIGNAL(clicked()),
	    this, SLOT(installShader()));
    shaderVLayout->addWidget(saveShaderButton);
     QLabel * installDone = new QLabel (tr("pas d'action"));
     installDone->setAlignment(Qt::AlignHCenter);
     connect(this, SIGNAL(installation(QString)),
	    installDone, SLOT(setText(QString)));

    shaderVLayout->addWidget(installDone);
    shaderGroup->setLayout(shaderVLayout);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(shaderGroup);
    mainLayout->addStretch(1);
    setLayout(mainLayout);
}


/**
 * @fn QString ShaderTab::FindShaderPath()
 * @brief slot pour rcuperer le chemin des shaders
*/

QString ShaderTab::FindShaderPath()
{
	QString shaderPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
         m_LEshader_Edit->setText(shaderPath);
         emit(installation(tr("Nouveau chemin de Shader")));
         return shaderPath;
}


/**
  * @fn void ShaderTab::installShader()
  * @brief slot pour l'installation d'un shader
*/
void ShaderTab::installShader()
{
  if(!m_LEshader_Edit->text().isEmpty()&&!m_LElink_Edit->text().isEmpty()&&!m_LEmi_Edit->text().isEmpty())
  {
    if(CParam::copyShader(m_LEshader_Edit->text(),m_LElink_Edit->text(),m_LEmi_Edit->text()))
    {
        emit(installation(tr("Installation effectue")));   
    }
    else
    {
        emit(installation(tr("Problme lors de l'installation")));       
    }
  }
  else
  {
      emit(installation(tr("Problme lors de l'installation : des paramtres ne sont pas donns.")));       
  }
}
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

ConfigTab::ConfigTab( QWidget *parent)
    : QWidget(parent)
{
 
	QGroupBox *mayaGroup = new QGroupBox(tr("Maya"));
	QGroupBox *mrGroup = new QGroupBox(tr("Mental Ray"));
	
	QGroupBox *ShaderGroup = new QGroupBox(tr("Shader MR"));
	
	QLabel *mayaLabel = new QLabel(tr("Chemin de Maya"));
	QLabel *mrLabel = new QLabel(tr("Chemin de Mental Ray"));
	QLabel *shaderLabel = new QLabel(tr("Donnez l'extension, le chemin et l'obligation pour chaque extension"));
	 
	//pour maya 
	m_LEmaya_Edit = new QLineEdit();
	m_LEmaya_Edit->setText(CParam::getMayaPath());
	QPushButton *mayaPath = new QPushButton("...");
	mayaPath->setMaximumSize(35,20);
	
	connect(mayaPath, SIGNAL(clicked()),
	    this, SLOT(mayaPath()));
	
	QHBoxLayout * mayaHLayout = new QHBoxLayout;
	mayaHLayout->addWidget(m_LEmaya_Edit);
	mayaHLayout->addWidget(mayaPath);
	
	QVBoxLayout *mayaVLayout = new QVBoxLayout;
	mayaVLayout->addWidget(mayaLabel);
	mayaVLayout->addLayout(mayaHLayout);
	
	mayaGroup->setLayout(mayaVLayout);
	 
	//pour Mental Ray 
	m_LEmr_Edit= new QLineEdit();
	m_LEmr_Edit->setText(CParam::getMRPath());
	QPushButton *mrPath = new QPushButton("...");
	mrPath->setMaximumSize(35,20);
	
	connect(mrPath, SIGNAL(clicked()),
	    this, SLOT(mrPath()));
	
	QHBoxLayout * mrHLayout = new QHBoxLayout;
	mrHLayout->addWidget(m_LEmr_Edit);
	mrHLayout->addWidget(mrPath);
	
	QVBoxLayout *mrVLayout = new QVBoxLayout;
	mrVLayout->addWidget(mrLabel);
	mrVLayout->addLayout(mrHLayout);
	
	mrGroup->setLayout(mrVLayout);
	 
	 
	//Pour les ext des shaders
	QPushButton *shaderButton[5];
	QHBoxLayout *shaderHLayout[5];
	QVBoxLayout *shaderVLayout= new QVBoxLayout;;
	int i;
	for (i =0 ;i <5 ;++i)
	{
		m_LEextPath_Edit[i]=new QLineEdit;
        m_LEextPath_Edit[i]->setText(CParam::getShaders()[0].getChemin(i).getPath());
		m_LEext_Edit[i] = new QLineEdit;
        m_LEext_Edit[i]->setText(CParam::getShaders()[0].getChemin(i).getExtension());
		shaderButton[i]=new QPushButton("...");
		shaderHLayout[i]= new QHBoxLayout;
		m_Cext_Mandatory[i] = new QComboBox();
		
		m_LEext_Edit[i]->setMaximumSize(45,20);
		m_LEext_Edit[i]->setMaxLength(7);
		shaderButton[i]->setMaximumSize(35,20);

		m_Cext_Mandatory[i]->addItem(tr("non"));
    	m_Cext_Mandatory[i]->addItem(tr("oui"));
    	m_Cext_Mandatory[i]->setEditable(false);
    	m_Cext_Mandatory[i]->setCurrentIndex(CParam::getShaders()[0].getChemin(i).isMandatory()*1);
		
		shaderHLayout[i]->addWidget(m_LEext_Edit[i]);
		shaderHLayout[i]->addWidget(m_LEextPath_Edit[i]);
		shaderHLayout[i]->addWidget(shaderButton[i]);
		shaderHLayout[i]->addWidget(m_Cext_Mandatory[i]);
		
		
		shaderVLayout->addWidget(shaderLabel);
		shaderVLayout->addLayout(shaderHLayout[i]);
		
		
	}
	
	connect(shaderButton[0], SIGNAL(clicked()),
	    		this, SLOT(shaderPath0()));
	connect(shaderButton[1], SIGNAL(clicked()),
	    		this, SLOT(shaderPath1()));
	connect(shaderButton[2], SIGNAL(clicked()),
	    		this, SLOT(shaderPath2()));
	connect(shaderButton[3], SIGNAL(clicked()),
	    		this, SLOT(shaderPath3()));
	connect(shaderButton[4], SIGNAL(clicked()),
	    		this, SLOT(shaderPath4()));	   
    connect(m_LEextPath_Edit[0], SIGNAL(textChanged ( QString )),
	    		this, SLOT(shaderPath0(QString)));
	connect(m_LEextPath_Edit[1], SIGNAL(textChanged ( QString )),
	    		this, SLOT(shaderPath1(QString)));
	connect(m_LEextPath_Edit[2], SIGNAL(textChanged ( QString )),
	    		this, SLOT(shaderPath2(QString)));
	connect(m_LEextPath_Edit[3], SIGNAL(textChanged ( QString )),
	    		this, SLOT(shaderPath3(QString)));
	connect(m_LEextPath_Edit[4], SIGNAL(textChanged ( QString )),
	    		this, SLOT(shaderPath4(QString)));	
                 		
	connect(m_Cext_Mandatory[0], SIGNAL(highlighted(QString)),
	    		this, SLOT(shaderMandatory0(QString)));	
	connect(m_Cext_Mandatory[1], SIGNAL(highlighted(QString)),
	    		this, SLOT(shaderMandatory1(QString)));	
	connect(m_Cext_Mandatory[2], SIGNAL(highlighted(QString)),
	    		this, SLOT(shaderMandatory2(QString)));	
	connect(m_Cext_Mandatory[3], SIGNAL(highlighted(QString)),
	    		this, SLOT(shaderMandatory3(QString)));	
	connect(m_Cext_Mandatory[4], SIGNAL(highlighted(QString)),
	    		this, SLOT(shaderMandatory4(QString)));	                                                	
	
	
	ShaderGroup->setLayout(shaderVLayout);	
	 
	 
    QPushButton * saveSettingsButton= new QPushButton(tr("Sauvegarde"));
    connect(saveSettingsButton, SIGNAL(clicked()),
	    this, SLOT(saveShader()));
    
	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget(mayaGroup);
	mainLayout->addWidget(mrGroup);
	mainLayout->addWidget(ShaderGroup);
	mainLayout->addWidget(saveSettingsButton);
	mainLayout->addStretch(1);
	setLayout(mainLayout);
 
}


/**
 * @fn QString ConfigTab::mayaPath()
 * @brief slot pour rcuperer le chemin de maya
*/

QString ConfigTab::mayaPath()
{
	QString mayaPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
         m_LEmaya_Edit->setText(mayaPath);
         CParam::writeMayaPath(mayaPath);
         return mayaPath;
}


/**
 * @fn QString ConfigTab::mrPath()
 * @brief slot pour rcuperer le chemin de Mental Ray
*/

QString ConfigTab::mrPath()
{
	QString mrPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
         m_LEmr_Edit->setText(mrPath);
         CParam::writeMentalRayPath(mrPath);
         return mrPath;
}

/**
 * @fn QString ConfigTab::shaderPath0()
 * @brief slot pour rcuperer le chemin de Shader
*/

QString ConfigTab::shaderPath0()
{
	QString shaderPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
     m_LEextPath_Edit[0]->setText(shaderPath);
     
     shaderPath0(shaderPath);
     
     return shaderPath;
}

/**
 * @fn QString ConfigTab::shaderPath1()
 * @brief slot pour rcuperer le chemin de Shader
*/

QString ConfigTab::shaderPath1()
{
	QString shaderPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
     m_LEextPath_Edit[1]->setText(shaderPath);
     shaderPath1(shaderPath);       
     return shaderPath;
}

/**
 * @fn QString ConfigTab::shaderPath2()
 * @brief slot pour rcuperer le chemin de Shader
*/

QString ConfigTab::shaderPath2()
{
	QString shaderPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
     m_LEextPath_Edit[2]->setText(shaderPath);
     shaderPath2(shaderPath);        
     return shaderPath;
}

/**
 * @fn QString ConfigTab::shaderPath3()
 * @brief slot pour rcuperer le chemin de Shader
*/

QString ConfigTab::shaderPath3()
{
	QString shaderPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
     m_LEextPath_Edit[3]->setText(shaderPath);
     shaderPath3(shaderPath);        
     return shaderPath;
}

/**
 * @fn QString ConfigTab::shaderPath4()
 * @brief slot pour rcuperer le chemin de Shader
*/

QString ConfigTab::shaderPath4()
{
	QString shaderPath= QFileDialog::getExistingDirectory(
                    this,
                    tr("Slectionner un rpertoire"),
                    ".");	
         m_LEextPath_Edit[4]->setText(shaderPath);
         
         shaderPath4(shaderPath);
         
         return shaderPath;
}

/**
 * @fn QString ConfigTab::shaderPath0(const QString &shaderPath)
 * @brief slot pour quand le chemin est chang par l'utilisateur
 *        c'est la ligne de fti qui est change
 *
*/
QString ConfigTab::shaderPath0(const QString &shaderPath)
{
     if(!m_LEext_Edit[0]->text().isEmpty() && !shaderPath.isEmpty())
     {
          CParam::setChemin(0,0,m_LEext_Edit[0]->text(),shaderPath,m_Cext_Mandatory[0]->currentIndex());
          
     }
     return shaderPath;        
}
/**
 * @fn QString ConfigTab::shaderPath0(const QString &shaderPath)
 * @brief slot pour quand le chemin est chang par l'utilisateur
 *        c'est la ligne de xpm qui est change
 *
*/
QString ConfigTab::shaderPath1(const QString &shaderPath)
{
     if(!m_LEext_Edit[1]->text().isEmpty() && !shaderPath.isEmpty())
     {
          CParam::setChemin(0,1,m_LEext_Edit[1]->text(),shaderPath,m_Cext_Mandatory[1]->currentIndex());
          
     }       
     return shaderPath;        
}
/**
 * @fn QString ConfigTab::shaderPath0(const QString &shaderPath)
 * @brief slot pour quand le chemin est chang par l'utilisateur
 *        c'est la ligne de mel qui est change
 *
*/
QString ConfigTab::shaderPath2(const QString &shaderPath)
{
     if(!m_LEext_Edit[2]->text().isEmpty() && !shaderPath.isEmpty())
     {
          CParam::setChemin(0,2,m_LEext_Edit[2]->text(),shaderPath,m_Cext_Mandatory[2]->currentIndex());
          
     }         
     return shaderPath;       
}
/**
 * @fn QString ConfigTab::shaderPath0(const QString &shaderPath)
 * @brief slot pour quand le chemin est chang par l'utilisateur
 *        c'est la ligne de mi qui est change
 *
*/
QString ConfigTab::shaderPath3(const QString &shaderPath)
{
     m_LEextPath_Edit[3]->setText(shaderPath);
     if(!m_LEext_Edit[3]->text().isEmpty() && !shaderPath.isEmpty())
     {
          CParam::setChemin(0,3,m_LEext_Edit[3]->text(),shaderPath,m_Cext_Mandatory[3]->currentIndex());
          
     }         
     return shaderPath;        
}
/**
 * @fn QString ConfigTab::shaderPath0(const QString &shaderPath)
 * @brief slot pour quand le chemin est chang par l'utilisateur
 *        c'est la ligne de dll qui est change
 *
*/
QString ConfigTab::shaderPath4(const QString &shaderPath)
{
    if(!m_LEext_Edit[4]->text().isEmpty() && !shaderPath.isEmpty())
    {
      CParam::setChemin(0,4,m_LEext_Edit[4]->text(),shaderPath,m_Cext_Mandatory[4]->currentIndex());
      
    }
    return shaderPath;        
}
///////////////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////////////
/**
 * @fn bool ConfigTab::shaderMandatory0(const QString &choice)
 * @brief slot pour le fti
*/
bool ConfigTab::shaderMandatory0(const QString &choice)
{
 return true;
}
/**
 * @fn bool ConfigTab::shaderMandatory0(const QString &choice)
 * @brief slot pour le xpm
*/
bool ConfigTab::shaderMandatory1(const QString &choice)
{
 return true;
}
/**
 * @fn bool ConfigTab::shaderMandatory0(const QString &choice)
 * @brief slot pour le mel
*/
bool ConfigTab::shaderMandatory2(const QString &choice)
{
 return true;
}
/**
 * @fn bool ConfigTab::shaderMandatory0(const QString &choice)
 * @brief slot pour le mi
*/
bool ConfigTab::shaderMandatory3(const QString &choice)
{
 return true;
}
/**
 * @fn bool ConfigTab::shaderMandatory0(const QString &choice)
 * @brief slot pour le dll
*/
bool ConfigTab::shaderMandatory4(const QString &choice)
{
 return true;
}


/**

*/
bool ConfigTab::saveShader()
{
 CParam::saveParam();
 return true;     
}





////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////



CreditTab::CreditTab( QWidget *parent )
{
 QTextEdit *credit = new QTextEdit();
 credit->setHtml(tr("<center>ShaderCopy V2.0 <p> Ralis par Franois DEVIC <p> <a href=\"http://www.devic.net\" >www.devic.net</a> <p> Maya&copy; est une marque dpose par <a href=\"http://www.alias.com\">Alias</a> (http://www.alias.com). <p> Mental Ray&copy; est une marque dpose par <a href=\"http://www.mentalimages.com/\">mental images</a> (http://www.mentalimages.com/). <p><p>Ce logiciel a t dveloppe  l'aide <b>QT 4 OpenSource</b> (http://www.trolltech.com).<p><b>ShaderCopy</b> version 2.0, Copyright (C) 2005 Francois DEVIC <b>ShaderCopy</b> comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions.</center>"));
 
 credit->setAlignment(Qt::AlignCenter);
// credit->setReadOnly(true);
// credit->setEnabled(false);
 QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget(credit);
	mainLayout->addStretch(1);
	setLayout(mainLayout);
}
